module CheckDiagram where
import System
import System.IO
import CCO.Component (Component, component, ioWrap, printer)
import CCO.Diag.Base
import CCO.Feedback
import CCO.Printing
import CCO.Tree (parser, ATerm, fromTree, toTree)
import CCO.Tree.Parser
import Control.Arrow  (Arrow (arr), (>>>))
import DiagramTypeChecker

-- consumes an ATerm as produced by the program parse-tdiag and typechecks the
-- represented diagram specifications, producing either descriptive error
-- messages for ill-typed specifications or else, for well-typed specifications,
-- just the ATerm it consumed

main = ioWrap(parser >>> aTerm2Diag >>> checkDiag >>> diag2ATerm >>> printer)

-- convert ATerm to Diag
aTerm2Diag :: Component ATerm Diag
aTerm2Diag = component (\inp -> do toTree inp)

-- type check Diag
checkDiag :: Component Diag Diag
checkDiag = component (\inp -> do tyCheck inp
                                  return inp)

-- run the type checker and produce feedback
tyCheck :: Diag -> Feedback DiagTy
tyCheck t = do let syn = wrap_Diag (sem_Diag t) (Inh_Diag liftingAllowed)
               messages [ Error (pp tyErr) | tyErr <- tyErrs_Syn_Diag syn]
               return (ty_Syn_Diag syn)
    where -- lifting of diagrams is allowed or not (see readme.txt)
          liftingAllowed = True

-- convert Diag to ATerm
diag2ATerm :: Component Diag ATerm
diag2ATerm = component (\inp -> do return $ fromTree inp)
