module Diagram2Picture where
import System
import System.IO
import CCO.Component (Component, component, ioWrap, printer)
import CCO.Diag.Base
import CCO.Feedback
import CCO.Picture
import CCO.Tree (parser, ATerm, fromTree, toTree)
import CCO.Tree.Parser
import Control.Arrow  (Arrow (arr), (>>>))
import DiagramTranslator

-- consumes an ATerm as produced by the programs parse-tdiag and tc-tdiag and
-- that translates the represented diagram specifications into LaTeX-code for
-- rendering the diagrams, producing an ATerm that describes the structure of
-- the generated LaTeX-code

main = ioWrap(parser >>> aTerm2Diag >>> diag2Picture >>> picture2ATerm >>> printer)

-- convert ATerm to Diag
aTerm2Diag :: Component ATerm Diag
aTerm2Diag = component (\inp -> do toTree inp)

-- convert Diag to Picture
diag2Picture :: Component Diag Picture
diag2Picture = component (\inp -> do return (translate inp))

-- run the translator
translate :: Diag -> Picture
translate t = let syn = wrap_Diag (sem_Diag t) (Inh_Diag "" 0 0 0)
              in Picture (width_Syn_Diag syn, height_Syn_Diag syn) (cmds_Syn_Diag syn)

-- convert Picture to ATerm
picture2ATerm :: Component Picture ATerm
picture2ATerm = component (\inp -> do return $ fromTree inp)
