

-- UUAGC 0.9.8 (..\src\DiagramTypeChecker.ag)
module DiagramTypeChecker where

{-# LINE 4 "..\\src\\DiagramTypeChecker.ag" #-}

import CCO.Diag.Base
import CCO.SourcePos
import CCO.Printing
{-# LINE 12 "..\\src\\DiagramTypeChecker.hs" #-}
{-# LINE 29 "..\\src\\DiagramTypeChecker.ag" #-}

type Indent = String

data DiagTy = RunnableTy
            | InterpreterTy Indent Indent
            | CompilerTy Indent Ident Indent
            | ProgramTy Indent
            | RunnableCompilerTy Indent Indent
            | PlatformTy Ident deriving (Eq, Show)

data DiagTyErr = DiagTyErr SourcePos
                           String
                           DiagTy
                           DiagTy

instance Printable DiagTyErr where pp = ppDiagTyErr

-- pretty print type errors
ppDiagTyErr :: DiagTyErr -> Doc
ppDiagTyErr (DiagTyErr pos err t1 t2)
  = (text $ showSPos pos)
    >-< (indent 2 ((text err)
    >-< (indent 2 ((text $ "expected type: " ++ showTy t2)
    >-<            (text $ "inferred type: " ++ showTy t1) ))))
  where showSPos (SourcePos s p) = showPos p
        showPos (Pos l c) = "line " ++ show l ++ ":column " ++ show c ++ ":"
        showPos (EOF)     = "End of file:"
        showTy (RunnableTy) = "Runnable"
        showTy (InterpreterTy l m) = "Interpreter for " ++ l ++ " on " ++ m
        showTy (CompilerTy l1 l2 m) = "Compiler from " ++ l1 ++ " to " ++ l2 ++ " on " ++ m
        showTy (ProgramTy m) = "Program on " ++ m
        showTy (RunnableCompilerTy l1 l2) = "RunnableCompiler"
        showTy (PlatformTy m) = "Platform offering " ++ m
{-# LINE 47 "..\\src\\DiagramTypeChecker.hs" #-}

{-# LINE 116 "..\\src\\DiagramTypeChecker.ag" #-}


checkTyExecute :: SourcePos -> Bool -> DiagTy -> DiagTy -> [DiagTyErr]
checkTyExecute pos lift t1 t2
  = case t2 of
         (PlatformTy m)           -> check t1 m
         (InterpreterTy l m)      -> if lift
                                        then check t1 l
                                        else [DiagTyErr pos msg_int t2 (PlatformTy m)]
         (CompilerTy _ _ _)       -> [DiagTyErr pos msg_comp t2 (PlatformTy "any")]
         (RunnableCompilerTy _ _) -> [DiagTyErr pos msg_comp t2 (PlatformTy "any")]
         (ProgramTy _)            -> [DiagTyErr pos msg_prog t2 (PlatformTy "any")]
         _                        -> [DiagTyErr pos msg_prog t2 (PlatformTy "any")]
    where check t m
            = case t of
                   (ProgramTy l)            -> if m == l
                                                  then []
                                                  else [DiagTyErr pos msg_match t1 (ProgramTy m)]
                   (InterpreterTy l m')     -> if m == m'
                                                  then []
                                                  else [DiagTyErr pos msg_match t1 (InterpreterTy l m)]
                   (CompilerTy l1 l2 m')    -> if m == m'
                                                  then []
                                                  else [DiagTyErr pos msg_match t1 (CompilerTy l1 l2 m)]
                   (RunnableCompilerTy l1 l2) -> [DiagTyErr pos msg_rcomp t1 (CompilerTy l1 l2 m)]
                   (RunnableTy)               -> [DiagTyErr pos msg_exec t1 (ProgramTy m)]
                   (PlatformTy _)             -> [DiagTyErr pos msg_plat t1 (ProgramTy m)]
          msg_match = "The languages do not match."
          msg_rcomp = "You cannot execute an already executing compiler."
          msg_exec  = "You cannot execute an executable construction."
          msg_plat  = "You cannot execute a platform."
          msg_int   = "The interpreter on which is executed, is itself not executable."
          msg_comp  = "You cannot execute on a compiler."
          msg_prog  = "You cannot execute on a program."

checkTyCompile :: SourcePos -> Bool -> DiagTy -> DiagTy -> [DiagTyErr]
checkTyCompile pos lift t1 t2
  = case t2 of
         (RunnableCompilerTy l1 l2) -> check t1 l1
         (CompilerTy l1 l2 m)       -> if lift
                                          then check t1 l1
                                          else [DiagTyErr pos msg_with_comp t2 (RunnableCompilerTy l1 l2)]
         (InterpreterTy _ _)        -> [DiagTyErr pos msg_with_int t2 (RunnableCompilerTy "any" "any")]
         (PlatformTy _)             -> [DiagTyErr pos msg_with_plat t2 (RunnableCompilerTy "any" "any")]
         (ProgramTy _)              -> [DiagTyErr pos msg_with_prog t2 (RunnableCompilerTy "any" "any")]
    where check t l1
            = case t of
                   (ProgramTy l)            -> if l == l1
                                                  then []
                                                  else [DiagTyErr pos msg_prog t1 (ProgramTy l1)]
                   (InterpreterTy l m')     -> if l1 == m'
                                                  then []
                                                  else [DiagTyErr pos msg_int t1 (InterpreterTy l l1)]
                   (CompilerTy l1' l2' m')  -> if l1 == m'
                                                  then []
                                                  else [DiagTyErr pos msg_comp t1 (CompilerTy l1' l2' l1)]
                   (RunnableCompilerTy l1' l2') -> [DiagTyErr pos msg_rcomp t1 (CompilerTy l1' l2' l1)]
                   (RunnableTy)                 -> [DiagTyErr pos msg_exec t1 (ProgramTy l1)]
                   (PlatformTy _)               -> [DiagTyErr pos msg_plat t1 (ProgramTy l1)]
          msg_with_comp = "The compiler used to compile with, is not executable itself."
          msg_with_int  = "You cannot compile with an interpreter."
          msg_with_plat = "You cannot compile with a platform."
          msg_with_prog = "You cannot compile with a program."
          msg_prog      = "The language of the program to compile does not match this compiler's input language."
          msg_int       = "The language of the interpreter to compile does not match this compiler's input language."
          msg_comp      = "The language of the compiler to compile does not match this compiler's input language."
          msg_rcomp     = "You cannot compile an already executing compiler."
          msg_exec      = "You cannot compile an executable construction."
          msg_plat      = "You cannot compile a platform."

{-# LINE 120 "..\\src\\DiagramTypeChecker.hs" #-}
-- Diag --------------------------------------------------------
-- cata
sem_Diag :: Diag  ->
            T_Diag 
sem_Diag (Diag _pos _d )  =
    (sem_Diag_Diag _pos (sem_Diag_ _d ) )
-- semantic domain
type T_Diag  = ( Bool ) ->
               ( ( SourcePos ),( DiagTy ),( [DiagTyErr] ))
data Inh_Diag  = Inh_Diag {lifting_Inh_Diag ::  Bool }
data Syn_Diag  = Syn_Diag {pos_Syn_Diag ::  SourcePos ,ty_Syn_Diag ::  DiagTy ,tyErrs_Syn_Diag ::  [DiagTyErr] }
wrap_Diag :: T_Diag  ->
             Inh_Diag  ->
             Syn_Diag 
wrap_Diag sem (Inh_Diag _lhsIlifting )  =
    (let ( _lhsOpos,_lhsOty,_lhsOtyErrs) =
             (sem _lhsIlifting )
     in  (Syn_Diag _lhsOpos _lhsOty _lhsOtyErrs ))
sem_Diag_Diag :: SourcePos ->
                 T_Diag_  ->
                 T_Diag 
sem_Diag_Diag pos_ d_  =
    (\ _lhsIlifting ->
         (let _dOpos :: ( SourcePos )
              _lhsOpos :: ( SourcePos )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _lhsOty :: ( DiagTy )
              _dOlifting :: ( Bool )
              _dIty :: ( DiagTy )
              _dItyErrs :: ( [DiagTyErr] )
              _dOpos =
                  {-# LINE 79 "..\\src\\DiagramTypeChecker.ag" #-}
                  pos_
                  {-# LINE 154 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOpos =
                  {-# LINE 80 "..\\src\\DiagramTypeChecker.ag" #-}
                  pos_
                  {-# LINE 158 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 69 "..\\src\\DiagramTypeChecker.ag" #-}
                  _dItyErrs
                  {-# LINE 162 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOty =
                  {-# LINE 70 "..\\src\\DiagramTypeChecker.ag" #-}
                  _dIty
                  {-# LINE 166 "..\\src\\DiagramTypeChecker.hs" #-}
              _dOlifting =
                  {-# LINE 67 "..\\src\\DiagramTypeChecker.ag" #-}
                  _lhsIlifting
                  {-# LINE 170 "..\\src\\DiagramTypeChecker.hs" #-}
              ( _dIty,_dItyErrs) =
                  (d_ _dOlifting _dOpos )
          in  ( _lhsOpos,_lhsOty,_lhsOtyErrs)))
-- Diag_ -------------------------------------------------------
-- cata
sem_Diag_ :: Diag_  ->
             T_Diag_ 
sem_Diag_ (Compile _d1 _d2 )  =
    (sem_Diag__Compile (sem_Diag _d1 ) (sem_Diag _d2 ) )
sem_Diag_ (Compiler _c _l1 _l2 _m )  =
    (sem_Diag__Compiler _c _l1 _l2 _m )
sem_Diag_ (Execute _d1 _d2 )  =
    (sem_Diag__Execute (sem_Diag _d1 ) (sem_Diag _d2 ) )
sem_Diag_ (Interpreter _i _l _m )  =
    (sem_Diag__Interpreter _i _l _m )
sem_Diag_ (Platform _m )  =
    (sem_Diag__Platform _m )
sem_Diag_ (Program _p _l )  =
    (sem_Diag__Program _p _l )
-- semantic domain
type T_Diag_  = ( Bool ) ->
                ( SourcePos ) ->
                ( ( DiagTy ),( [DiagTyErr] ))
data Inh_Diag_  = Inh_Diag_ {lifting_Inh_Diag_ ::  Bool ,pos_Inh_Diag_ ::  SourcePos }
data Syn_Diag_  = Syn_Diag_ {ty_Syn_Diag_ ::  DiagTy ,tyErrs_Syn_Diag_ ::  [DiagTyErr] }
wrap_Diag_ :: T_Diag_  ->
              Inh_Diag_  ->
              Syn_Diag_ 
wrap_Diag_ sem (Inh_Diag_ _lhsIlifting _lhsIpos )  =
    (let ( _lhsOty,_lhsOtyErrs) =
             (sem _lhsIlifting _lhsIpos )
     in  (Syn_Diag_ _lhsOty _lhsOtyErrs ))
sem_Diag__Compile :: T_Diag  ->
                     T_Diag  ->
                     T_Diag_ 
sem_Diag__Compile d1_ d2_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _d1Olifting :: ( Bool )
              _d2Olifting :: ( Bool )
              _d1Ipos :: ( SourcePos )
              _d1Ity :: ( DiagTy )
              _d1ItyErrs :: ( [DiagTyErr] )
              _d2Ipos :: ( SourcePos )
              _d2Ity :: ( DiagTy )
              _d2ItyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 98 "..\\src\\DiagramTypeChecker.ag" #-}
                  case _d2Ity of
                       (RunnableCompilerTy l1 l2) -> case _d1Ity of
                                                          (ProgramTy l)          -> ProgramTy l2
                                                          (InterpreterTy l m)    -> InterpreterTy l l2
                                                          (CompilerTy l1' l2' m) -> CompilerTy l1' l2' l2
                                                          _                      -> RunnableTy
                       (CompilerTy l1 l2 m)       -> case _d1Ity of
                                                          (ProgramTy l)          -> ProgramTy l2
                                                          (InterpreterTy l m)    -> InterpreterTy l l2
                                                          (CompilerTy l1' l2' m) -> CompilerTy l1' l2' l2
                                                          _                      -> RunnableTy
                       _                          -> RunnableTy
                  {-# LINE 233 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 110 "..\\src\\DiagramTypeChecker.ag" #-}
                  checkTyCompile _d1Ipos _lhsIlifting _d1Ity _d2Ity
                  {-# LINE 237 "..\\src\\DiagramTypeChecker.hs" #-}
              _d1Olifting =
                  {-# LINE 67 "..\\src\\DiagramTypeChecker.ag" #-}
                  _lhsIlifting
                  {-# LINE 241 "..\\src\\DiagramTypeChecker.hs" #-}
              _d2Olifting =
                  {-# LINE 67 "..\\src\\DiagramTypeChecker.ag" #-}
                  _lhsIlifting
                  {-# LINE 245 "..\\src\\DiagramTypeChecker.hs" #-}
              ( _d1Ipos,_d1Ity,_d1ItyErrs) =
                  (d1_ _d1Olifting )
              ( _d2Ipos,_d2Ity,_d2ItyErrs) =
                  (d2_ _d2Olifting )
          in  ( _lhsOty,_lhsOtyErrs)))
sem_Diag__Compiler :: Ident ->
                      Ident ->
                      Ident ->
                      Ident ->
                      T_Diag_ 
sem_Diag__Compiler c_ l1_ l2_ m_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 89 "..\\src\\DiagramTypeChecker.ag" #-}
                  CompilerTy l1_ l2_ m_
                  {-# LINE 264 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 90 "..\\src\\DiagramTypeChecker.ag" #-}
                  []
                  {-# LINE 268 "..\\src\\DiagramTypeChecker.hs" #-}
          in  ( _lhsOty,_lhsOtyErrs)))
sem_Diag__Execute :: T_Diag  ->
                     T_Diag  ->
                     T_Diag_ 
sem_Diag__Execute d1_ d2_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _d1Olifting :: ( Bool )
              _d2Olifting :: ( Bool )
              _d1Ipos :: ( SourcePos )
              _d1Ity :: ( DiagTy )
              _d1ItyErrs :: ( [DiagTyErr] )
              _d2Ipos :: ( SourcePos )
              _d2Ity :: ( DiagTy )
              _d2ItyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 91 "..\\src\\DiagramTypeChecker.ag" #-}
                  case _d1Ity of
                       (ProgramTy l)        -> RunnableTy
                       (InterpreterTy l m)  -> PlatformTy l
                       (CompilerTy l1 l2 m) -> RunnableCompilerTy l1 l2
                       _                    -> RunnableTy
                  {-# LINE 293 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 97 "..\\src\\DiagramTypeChecker.ag" #-}
                  checkTyExecute _d1Ipos _lhsIlifting _d1Ity _d2Ity
                  {-# LINE 297 "..\\src\\DiagramTypeChecker.hs" #-}
              _d1Olifting =
                  {-# LINE 67 "..\\src\\DiagramTypeChecker.ag" #-}
                  _lhsIlifting
                  {-# LINE 301 "..\\src\\DiagramTypeChecker.hs" #-}
              _d2Olifting =
                  {-# LINE 67 "..\\src\\DiagramTypeChecker.ag" #-}
                  _lhsIlifting
                  {-# LINE 305 "..\\src\\DiagramTypeChecker.hs" #-}
              ( _d1Ipos,_d1Ity,_d1ItyErrs) =
                  (d1_ _d1Olifting )
              ( _d2Ipos,_d2Ity,_d2ItyErrs) =
                  (d2_ _d2Olifting )
          in  ( _lhsOty,_lhsOtyErrs)))
sem_Diag__Interpreter :: Ident ->
                         Ident ->
                         Ident ->
                         T_Diag_ 
sem_Diag__Interpreter i_ l_ m_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 87 "..\\src\\DiagramTypeChecker.ag" #-}
                  InterpreterTy l_ m_
                  {-# LINE 323 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 88 "..\\src\\DiagramTypeChecker.ag" #-}
                  []
                  {-# LINE 327 "..\\src\\DiagramTypeChecker.hs" #-}
          in  ( _lhsOty,_lhsOtyErrs)))
sem_Diag__Platform :: Ident ->
                      T_Diag_ 
sem_Diag__Platform m_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 85 "..\\src\\DiagramTypeChecker.ag" #-}
                  PlatformTy m_
                  {-# LINE 339 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 86 "..\\src\\DiagramTypeChecker.ag" #-}
                  []
                  {-# LINE 343 "..\\src\\DiagramTypeChecker.hs" #-}
          in  ( _lhsOty,_lhsOtyErrs)))
sem_Diag__Program :: Ident ->
                     Ident ->
                     T_Diag_ 
sem_Diag__Program p_ l_  =
    (\ _lhsIlifting
       _lhsIpos ->
         (let _lhsOty :: ( DiagTy )
              _lhsOtyErrs :: ( [DiagTyErr] )
              _lhsOty =
                  {-# LINE 83 "..\\src\\DiagramTypeChecker.ag" #-}
                  ProgramTy l_
                  {-# LINE 356 "..\\src\\DiagramTypeChecker.hs" #-}
              _lhsOtyErrs =
                  {-# LINE 84 "..\\src\\DiagramTypeChecker.ag" #-}
                  []
                  {-# LINE 360 "..\\src\\DiagramTypeChecker.hs" #-}
          in  ( _lhsOty,_lhsOtyErrs)))